﻿//http://forums.xna.com/forums/t/32476.aspx
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace _297Final
{
    // This is a basic class to animate a sprite sheet.
    // I've pretty much hard-coded this to use a 5x5 sheet of 64x64 sprites, so if you
    // want to use this for something else, then you'll have to modify it appropriately.
    //
    // Hardcoding like this isn't what I would consider good programming practice as it
    // can't be reused easily, but this class is designed specifically to display sheets
    // created with the explosion generator, so I'm giving myself a pass on this one.
    // It's simple enough that you should be able to create your own the "right" way.
    public class Animation
    {
        public Texture2D Texture { get; set; }  // The sprite sheet image
        public bool Finished { get; set; }      // notes if the animation has run completely
        public float FPS = 1000.0f / 30.0f;     // Frames per second of this animation
                                                // 1000 represents one milisecond
                                                // 30 represents how many frames we want to display in 1000 miliseconds
                                                // I could've used 1.0f / 30.0f if I wanted to change the Update()
                                                // code to count .TotalSeconds instead of Milliseconds.
                                                // The net result here is that it should play 30 frames per second.
        public Rectangle[] frames;              // Array of rectangles to represent each frame of the animation sequence
        public int currentFrame = 0;            // The frame that should currently be displayed
        private float frameTimer = 0.0f;        // A counter to keep track of how long since the frame's been changed

        // TODO:  Require the texture
        // Standard constructor...
        public Animation()
        {
            // Build the frame rectangles that represent each frame of the animation sequence
            // going from left to right, top to bottom
            frames = new Rectangle[25];
            for (int y = 0; y < 5; y++)
            {
                for (int x = 0; x < 5; x++)
                {
                    frames[x + y * 5] = new Rectangle(x * 64, y * 64, 64, 64);
                }
            }
            Finished = false;   //initially the animation hasn't run
        }

        // This method will change the current frame when necessary
        public void Update(GameTime gameTime)
        {
            // How many milliseconds since the last time the game's Update() method was called
            float elapsedTime = (float)gameTime.ElapsedGameTime.TotalMilliseconds;

            // Accumulate time this frame has been active
            frameTimer += elapsedTime;

            // If the time active for this frame is more than the time allowed...
            if (frameTimer >= FPS)
            {
                // Reset the frame timer
                frameTimer = 0;

                // Increment the current frame, then test it against the number of frames available...
                if (++currentFrame >= frames.Length)
                {
                    // Reset the frame to the beginning when it's over
                    currentFrame = 0;
                    Finished = true;
                }
            }
        }

        // Basic method to draw the current frame
        public void Draw(SpriteBatch spriteBatch, Vector2 position, float size)
        {
            if(!Finished)   //if the animation hasn't completely run
                spriteBatch.Draw(Texture, position, frames[currentFrame], Color.White, 0.0f, Vector2.Zero, size, SpriteEffects.None, 0.0f);
        }
    }
}
