using System;
using System.Collections.Generic;
using System.Linq;
using System.Diagnostics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace _297Final
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        //graphics
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        ScrollingBackground background;
        SpriteFont font;
        
        //audio
        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;
        
        //game objects
        Spaceship ywing = new Spaceship();  //spaceship class
        FadeSprite continueSprite = new FadeSprite();   //holds the fade in/fade out continue text
        Texture2D TitleScreen, EndScreen;   //Title screen Tex
        Texture2D bomb;                     //bomb tex
        Texture2D[] planets;                //array of planet textures
        Texture2D[] explosions;             //array of ship explosions
        Matrix SpriteScale;                 //used to scale sprites to correct size
        List<Planet> pList = new List<Planet>();     //queue to hold the planets
        Animation explode;                  //explosion animation
        Animation planExplode;              //planet explosion
        Bomb pickup,bullet;                //bullet class
        
        //states
        GameState gameState;                //current game state
        PlayerState pState;                 //current player state
        
        //global variables
        int xMin, xMax, yMin, yMax;         //limits of the screen width/height
        float moveSpeed;                    //speed the ship can usually move
        float bgScroll;                     //background scroll speed
        int lives;                          //lives the player has
        int distance;                       //distance the player has traveled
        int highScore;                      //highest score recorded this playthrough
        bool planetExplode;                 //is a planet exploding?
        float explTime;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            Vector2 shipPos = new Vector2(100, 540);
            ywing.Position = shipPos;       //initial start of the spaceship
            ywing.Bombs = 0;                //ships starts with no bombs
            gameState = GameState.Title;    //start at the title screen
            pState = PlayerState.Alive;     //player starts alive
            lives = 2;                      //player starts with 2 lives
            distance = 0;                   //initially player has gone 0 distance
            bgScroll = 0;                   //background scroll speed
            highScore = 0;                  //highscore starts at 0
                        
            continueSprite.Alpha = 1;       //alpha value of the fading text
            continueSprite.Increment = 10;   //amount to increment the fade text alpha
            continueSprite.Delay = 0.035f;  //delay between alpha changes
            continueSprite.IsVisible = true;    //initially we're on the title screen, this is visible

            //default resolution is 720p
            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            xMin = 50;
            xMax = 1800;
            yMin = 0;
            yMax = 980;

            //set planet spawn time
            Planet.Time = 1000; //planet hasn't been spawned yet
            planetExplode = false;
            explTime = 10000;

            //create texture arrays
            planets = new Texture2D[4];
            explosions = new Texture2D[6];
            
            //default bullet class
            bullet = new Bomb();
            bullet.Pos = new Vector2(-1000, -1000);
            bullet.IsVisible = false;
            bullet.Border = new Circle(bullet.Pos, 0);
            pickup = new Bomb();
            pickup = bullet;

            base.Initialize();
        }

        protected override void LoadContent()
        {
            //calculate scale ratio
            float screenScaleW = graphics.GraphicsDevice.Viewport.Width / 1920f;
            float screenScaleH = graphics.GraphicsDevice.Viewport.Height / 1080f;
            SpriteScale = Matrix.CreateScale(screenScaleW, screenScaleH, 1);

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            background = new ScrollingBackground();

            //load textures
            Texture2D bgTex = Content.Load<Texture2D>("starfieldBig");
            background.Load(GraphicsDevice, bgTex);
            ywing.Tex = Content.Load<Texture2D>("ywingshadow");
            TitleScreen = Content.Load<Texture2D>("TitleScreen");
            EndScreen = Content.Load<Texture2D>("GameOver");
            continueSprite.Texture = Content.Load<Texture2D>("ContinueSprite");
            planets[0] = Content.Load<Texture2D>("planet-001");
            planets[1] = Content.Load<Texture2D>("planet-002");
            planets[2] = Content.Load<Texture2D>("planet-003");
            planets[3] = Content.Load<Texture2D>("planet-004");
            explosions[0] = Content.Load<Texture2D>("explosion0");
            explosions[1] = Content.Load<Texture2D>("explosion1");
            explosions[2] = Content.Load<Texture2D>("explosion2");
            explosions[3] = Content.Load<Texture2D>("explosion3");
            explosions[4] = Content.Load<Texture2D>("explosion4");
            explosions[5] = Content.Load<Texture2D>("explosion5");
            font = Content.Load<SpriteFont>("Arial");    //load the font
            bomb = Content.Load<Texture2D>("banzai");

            //load sounds
            audioEngine = new AudioEngine("Content/Sounds.xgs");
            waveBank = new WaveBank(audioEngine, "Content/Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content/Sound Bank.xsb");

            //continue sprite location
            float continueX = (graphics.GraphicsDevice.Viewport.Width / 2) + (continueSprite.Texture.Width / 2);
            float continueY = (graphics.GraphicsDevice.Viewport.Height / 2) + (continueSprite.Texture.Height / 2);
            Rectangle contSpritePos = new Rectangle((int)continueX, (int)continueY, continueSprite.Texture.Width,
                                                    continueSprite.Texture.Height);
            continueSprite.Position = contSpritePos;    //location of continueSprite
            
            //ywing bounding box
            ywing.Border = new Rectangle((int)ywing.Position.X, (int)ywing.Position.Y, ywing.Tex.Width, ywing.Tex.Height);
        }


        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        GamePadState currentState = GamePad.GetState(PlayerIndex.One);
        protected override void Update(GameTime gameTime)
        {
            moveSpeed = 800 * (float)gameTime.ElapsedGameTime.TotalMilliseconds / 1000.0f; //move speed
            GamePadState prvState = currentState;
            currentState = GamePad.GetState(PlayerIndex.One);
            explTime += gameTime.ElapsedGameTime.Milliseconds;
            if (explTime > 1000) planetExplode = false;
            if (planetExplode) planExplode.Update(gameTime);
            
            //game exit code
            if (currentState.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            //continue text fade
            if (continueSprite.IsVisible)
            {
                continueSprite.Delay -= (float)gameTime.ElapsedGameTime.TotalSeconds;
                if (continueSprite.Delay < 0)
                {
                    continueSprite.Delay = 0.035f;
                    continueSprite.Alpha += continueSprite.Increment;
                    if ((continueSprite.Alpha > 255) || (continueSprite.Alpha < 0))
                        continueSprite.Increment *= -1; //if we've passed full fade in/out, reverse fade
                }
            }

            //go from title screen to gameplay
            if (gameState == GameState.Title)
            {
                lives = 2;
                distance = 0;
                pState = PlayerState.Spawning;
                ywing.DeathTime = 3000; //quick patch for respawn timer

                if ((currentState.Buttons.A == ButtonState.Released) &&
                    (prvState.Buttons.A == ButtonState.Pressed))
                {
                    soundBank.PlayCue("beep_02");
                    gameState = GameState.Game;
                    continueSprite.IsVisible = false;   //continue class no longer visible
                }
            }

            //go from end screen to title screen
            if (gameState == GameState.EndGame)
            {
                continueSprite.IsVisible = true;
                if ((currentState.Buttons.A == ButtonState.Released) &&
                    (prvState.Buttons.A == ButtonState.Pressed))
                {
                    gameState = GameState.Title;
                    soundBank.PlayCue("beep_02");
                }
            }

            //update bomb sprites
            if (pickup.IsVisible)
            {
                pickup.Pos = new Vector2(pickup.Pos.X + pickup.Speed, pickup.Pos.Y);
                pickup.Border = new Circle(pickup.Pos, bomb.Width);
                if (pickup.Pos.X < 0) pickup.IsVisible = false;
            }
            if (bullet.IsVisible)
            {
                bullet.Pos = new Vector2(bullet.Pos.X + bullet.Speed, bullet.Pos.Y);
                bullet.Border = new Circle(bullet.Pos, bomb.Width);
                if (bullet.Pos.X > 2000) bullet.IsVisible = false;
            }

            //Control ship movement
            if (gameState == GameState.Game)
            {
                //if player is alive, update the ship
                if ((pState == PlayerState.Alive) || pState == (PlayerState.Shield))
                {
                    distance++; //increment the distance
                    if (distance > highScore) highScore = distance; //update high score if necessary
                    Vector2 shipPos = new Vector2(ywing.Position.X, ywing.Position.Y);
                    shipPos.X += moveSpeed * currentState.ThumbSticks.Left.X;
                    shipPos.Y -= moveSpeed * currentState.ThumbSticks.Left.Y;
                    if (shipPos.X > xMax) shipPos.X = xMax;
                    if (shipPos.Y > yMax) shipPos.Y = yMax;
                    if (shipPos.X < xMin) shipPos.X = xMin;
                    if (shipPos.Y < yMin) shipPos.Y = yMin;
                    ywing.Position = shipPos;
                    ywing.Border = new Rectangle((int)ywing.Position.X, (int)ywing.Position.Y, ywing.Tex.Width, ywing.Tex.Height);
                    if ((distance % 1000 == 0) && (!pickup.IsVisible))
                        pickup = GameFunc.SpawnBomb(bomb,-moveSpeed);
                    if (pickup.Border.Intersects(ywing.Border))
                    {
                        ywing.Bombs++;
                        pickup.Pos = new Vector2(-1000, -1000);
                        pickup.Border = new Circle(pickup.Pos, 0);
                        pickup.IsVisible = false;
                    }
                    if (currentState.IsButtonUp(Buttons.RightTrigger) && prvState.IsButtonDown(Buttons.RightTrigger) && (ywing.Bombs>0))
                    {
                        bullet = GameFunc.SpawnBomb(bomb, 2 * moveSpeed);
                        bullet.Pos = ywing.Position;
                        bullet.Border = new Circle(bullet.Pos, bomb.Width);
                        ywing.Bombs--;
                        bullet.IsVisible = true;
                    }
                }

                //if player is dead, update the explosion
                if (pState == PlayerState.Dead)
                {
                    ywing.DeathTime += gameTime.ElapsedGameTime.Milliseconds;
                    explode.Update(gameTime);

                    if (ywing.DeathTime > 1000)    //if 3 seconds have passed since player death
                    {
                        lives--;    //remove a life
                        if (lives < 0)
                            gameState = GameState.EndGame;
                        else
                        {
                            pState = PlayerState.Spawning;
                            bgScroll = 0;
                        }
                        ywing.Position = new Vector2(100, 540); //respawn the player
                        GamePad.SetVibration(PlayerIndex.One, 0f, 0f);  //switch off vibration after 1 second
                    }
                }

                //if player respawn time is up, remove their invulnerability
                if (pState == PlayerState.Shield)
                {
                    ywing.DeathTime += gameTime.ElapsedGameTime.Milliseconds;
                    if (ywing.DeathTime > 4000)
                        pState = PlayerState.Alive;
                }

                //spawn a planet, if necessary
                Planet.Time += (float)gameTime.ElapsedGameTime.TotalMilliseconds;
                if ((pList.Count < 3) && (Planet.Time > 500))
                {
                    Random rand = new Random();
                    Planet newPlanet = new Planet();
                    newPlanet = GameFunc.SpawnPlanet(planets,distance);
                    pList.Add(newPlanet);
                    Planet.Time = 0 - rand.Next(0,500);
                }

                if (pState != PlayerState.Spawning)
                {
                    //update the scrolling background
                    bgScroll = (float)Math.Log(distance) - 1;
                    background.Update((float)gameTime.ElapsedGameTime.TotalSeconds*150*bgScroll);
                
                    //update planet locations
                    for(int i=0; i<pList.Count-1; i++)
                    {
                        Vector2 newPos = new Vector2(pList[i].Pos.X - moveSpeed * pList[i].Speed.X * 0.05f,
                                                     pList[i].Pos.Y + moveSpeed * pList[i].Speed.Y * 0.05f);
                        pList[i].Pos = newPos;
                        Circle newCir = new Circle();
                        newCir.Center.X = (newPos.X + 512 * pList[i].Size);
                        newCir.Center.Y = (newPos.Y + 512 * pList[i].Size);
                        newCir.Radius = 512 * pList[i].Size;
                        pList[i].PCircle = newCir;
                        pList[i].Angle += pList[i].RotateSpd;
                        if (bullet.IsVisible)
                        {
                            if (bullet.Border.Intersects(pList[i].PCircle))
                            {
                                soundBank.PlayCue("explosion");
                                bullet.IsVisible = false;
                                pList.Remove(pList[i]);
                                planExplode = new Animation();
                                Random rand = new Random();
                                planExplode.Texture = explosions[rand.Next(0, 6)];
                                planetExplode = true;
                                explTime = 0;
                            }
                        }
                        for (int j = 0; j < pList.Count-1; j++)
                        {
                            if ((pList[i] != pList[j]) && pList[i].PCircle.Intersects(pList[j].PCircle))    //planet collisions
                            {
                                float oldP2XSpeed, oldP2YSpeed;

                                oldP2XSpeed = pList[j].Speed.X;
                                oldP2YSpeed = pList[j].Speed.Y;
                                if (((pList[i].Speed.Y > 0) && (pList[j].Speed.Y > 0)) ||  //if both y values are the same direction
                                   ((pList[i].Speed.Y < 0) && (pList[j].Speed.Y < 0)))
                                    oldP2YSpeed *= -1;                      //reverse the y values on impact

                                if (pList[i].Speed.X == oldP2XSpeed) oldP2XSpeed *= 0.8f; //if the planets are going to stick, unstick them
                                
                                pList[j].Speed = new Vector2(pList[i].Speed.X, pList[j].Speed.Y);
                                pList[i].Speed = new Vector2(oldP2XSpeed, oldP2YSpeed);
                                
                            }
                        }
                        if ((pList[i].Pos.Y < yMin) || (pList[i].Pos.Y > yMax)) pList[i].Speed = new Vector2(pList[i].Speed.X, -1 * pList[i].Speed.Y);    //if we hit the top/bottom borders, reverse y direction
                        if (pList[i].PCircle.Intersects(ywing.Border) && (pState == PlayerState.Alive))
                        {
                            soundBank.PlayCue("explosion");
                            GamePad.SetVibration(PlayerIndex.One, 1.0f, 1.0f);
                            pState = PlayerState.Dead;  //player is dead
                            explode = new Animation();
                            Random rand = new Random();
                            explode.Texture = explosions[rand.Next(0, 6)];
                            ywing.DeathTime = 0;
                        }
                    }
                    Planet temp = new Planet();
                    temp = pList[0];
                    if ((temp.Pos.X < xMin - 500) || (temp.Pos.X > xMin + 3000)) pList.RemoveAt(0);
                }
                else
                {
                    for (int i = 0; i < pList.Count; i++)
                        pList[i].Pos = new Vector2(pList[i].Pos.X + 10, pList[i].Pos.Y);
                    distance -= 1;
                    if (distance < 0) distance = 0;
                    ywing.DeathTime += gameTime.ElapsedGameTime.Milliseconds;
                    if (ywing.DeathTime > 3000)
                        pState = PlayerState.Shield;
                }

            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.None, SpriteScale);
            switch (gameState)
            {
                case GameState.Title:
                    spriteBatch.Draw(TitleScreen, Vector2.Zero, Color.White);
                    spriteBatch.Draw(continueSprite.Texture, continueSprite.Position, new Color(255, 255, 255,
                                     (byte)MathHelper.Clamp(continueSprite.Alpha, 0, 255)));
                    break;
                case GameState.EndGame:
                    spriteBatch.Draw(EndScreen, Vector2.Zero, Color.White);
                    spriteBatch.Draw(continueSprite.Texture, continueSprite.Position, new Color(255, 255, 255,
                                     (byte)MathHelper.Clamp(continueSprite.Alpha, 0, 255)));
                    string hText = "High Score: " + highScore.ToString();    //high score string
                    spriteBatch.DrawString(font, hText, new Vector2(650, 500), Color.Yellow,
                                   0, Vector2.Zero, 4.0f, SpriteEffects.None, 0.5f);
                    break;
                case GameState.Game:
                    background.Draw(spriteBatch);   //draw the background
                    if (pickup.IsVisible)             //draw the bomb if visible
                        spriteBatch.Draw(bomb, pickup.Pos, Color.White);
                    if (bullet.IsVisible)             //draw the bomb if visible
                        spriteBatch.Draw(bomb, bullet.Pos, Color.White);
                    if(planetExplode)
                        planExplode.Draw(spriteBatch, bullet.Pos, 5f);
                    foreach (Planet p in pList)    //draw each planet
                    {
                        spriteBatch.Draw(p.Tex,p.Pos,null,Color.White,0,Vector2.Zero, p.Size, SpriteEffects.None,0);
                    }

                    string dText = "Distance: " + distance.ToString();  //distance info string
                    spriteBatch.DrawString(font, dText, new Vector2(100,50), Color.Yellow,
                                   0, Vector2.Zero, 3.0f, SpriteEffects.None, 0.5f);
                    string lText = "Lives: " + lives.ToString();    //life info string
                    spriteBatch.DrawString(font, lText, new Vector2(1600,50), Color.Yellow,
                                   0, Vector2.Zero, 3.0f, SpriteEffects.None, 0.5f);
                    hText = "High Score: " + highScore.ToString();    //high score string
                    spriteBatch.DrawString(font, hText, new Vector2(600, 50), Color.Yellow,
                                   0, Vector2.Zero, 3.0f, SpriteEffects.None, 0.5f);
                    string bText = "Bombs: " + ywing.Bombs.ToString();    //bomb info string
                    spriteBatch.DrawString(font, bText, new Vector2(1200, 50), Color.Yellow,
                                   0, Vector2.Zero, 3.0f, SpriteEffects.None, 0.5f);

                    switch (pState) //determine what to draw based on player state
                    {
                        case PlayerState.Alive:
                            spriteBatch.Draw(ywing.Tex, ywing.Position,null,Color.White,0,Vector2.Zero,1.25f,SpriteEffects.None,0);
                            break;
                        case PlayerState.Shield:
                            spriteBatch.Draw(ywing.Tex, ywing.Position, null, Color.LightBlue, 0, Vector2.Zero, 1.25f, SpriteEffects.None, 0);
                            break;
                        case PlayerState.Dead:
                            explode.Draw(spriteBatch,ywing.Position,3.5f);
                            break;
                        case PlayerState.Spawning:
                            break;
                    }
                    break;

            }
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
